require "spec_helper"

describe QueryInterface::Server::Resource, type: :controller do

  class Foo
  end

  controller(ActionController::Base) do
    include QueryInterface::Server::Resource.for(Foo)
  end

  context 'self.for' do
    it 'defines a query_model method' do
      subject.query_model.should eq(Foo)
    end

    it 'includes the instance methods' do
      subject.should respond_to(:query)
    end
  end

  context "GET 'query'" do
    before(:each) do
      routes.draw { get "query" => "anonymous#query" }
    end

    it 'runs the transformations' do
      dataset = double('dataset')
      transformer = double('transformer')
      result = double('result')
      Foo.should_receive(:filter).and_return(dataset)
      QueryInterface::Server::Transformations::SequelTransformer.should_receive(:new).with(dataset).and_return(transformer)
      transformer.should_receive(:run).with([{'my' => 'transformations'}]).and_return(result)
      get :query, transformations: [{'my' => 'transformations'}], format: :json
      response.should be_success
    end

    it 'returns 404 when result is nil' do
      dataset = double('dataset')
      transformer = double('transformer')
      Foo.should_receive(:filter).and_return(dataset)
      QueryInterface::Server::Transformations::SequelTransformer.should_receive(:new).with(dataset).and_return(transformer)
      transformer.should_receive(:run).with([{'my' => 'transformations'}]).and_return(nil)
      get :query, transformations: [{'my' => 'transformations'}], format: :json
      response.should be_not_found
    end
  end

end
