require "spec_helper"

describe QueryInterface::Server::Transformations do
  class Foo
    # TODO: Fiddle out a way to move this ONE dependency to Sequel somewhere else
    def self.dataset_module(&block)
    end
    include QueryInterface::Server::Transformations

    def self.model
      self
    end

    def self.table_name
      'foo_table'
    end
  end

  context "on include" do
    let(:lazy_query) { double("lazyquery") }

    it "adds a query_transformations method to the class" do
      Foo.should respond_to(:query_transformations)
    end

    it "adds a query method to the class" do
      Foo.should respond_to(:query)
    end

    it "should initialize a new LazyQuery object upon calling query" do
      QueryInterface::Server::LazyQuery.should_receive(:new).with(Foo).and_return(lazy_query)
      Foo.query.should eq(lazy_query)
    end

    it "adds a @query_transformations hash as class instance variable" do
      Foo.instance_variable_get(:@query_transformations).should eq({
        :exclude=>{}, :filter=>{}, :with=>{}, :order=>{}, :update=>{}
      })
    end
  end

  context "adding filter/with/order" do
    it "adds auto filter methods as stored procs" do
      Foo.query_transformations do
        auto_filter :foo, :bar
      end
      transformations = Foo.instance_variable_get(:@query_transformations)
      transformations[:filter].keys.should eq([:foo, :bar])
      transformations[:filter][:foo].should be_a(Proc)
      Foo.should_receive(:filter).with(Sequel.qualify('foo_table', :bar) => 'bar_value')
      Foo.instance_exec('bar_value', &transformations[:filter][:bar])
    end

    it "adds custom filters" do
      Foo.query_transformations do
        filter :foobar do |param|
          filter(foobar: param)
        end
      end
      transformations = Foo.instance_variable_get(:@query_transformations)
      transformations[:filter].keys.should eq([:foo, :bar, :foobar])
      transformations[:filter][:foobar].should be_a(Proc)
      Foo.should_receive(:filter).with(foobar: 'foo_bar_value')
      Foo.instance_exec('foo_bar_value', &transformations[:filter][:foobar])
    end

    it "adds custom withs" do
      Foo.query_transformations do
        with :thingies do
          join(:thingies_table)
        end
      end
      transformations = Foo.instance_variable_get(:@query_transformations)
      transformations[:with].keys.should eq([:thingies])
      transformations[:with][:thingies].should be_a(Proc)
      Foo.should_receive(:join).with(:thingies_table)
      Foo.instance_exec(&transformations[:with][:thingies])
    end

    it "adds custom order" do
      Foo.query_transformations do
        order :order_field do |direction|
          order(:order_field_thingy, direction)
        end
      end
      transformations = Foo.instance_variable_get(:@query_transformations)
      transformations[:order].keys.should eq([:order_field])
      transformations[:order][:order_field].should be_a(Proc)
      Foo.should_receive(:order).with(:order_field_thingy, :asc)
      Foo.instance_exec(:asc, &transformations[:order][:order_field])
    end
  end
end
